import java.io.IOException;
import java.io.InputStream;
import java.util.UUID;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import org.junit.BeforeClass;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.openkm.sdk4j.OKMWebservices;
import com.openkm.sdk4j.OKMWebservicesFactory;
import com.openkm.sdk4j.exception.*;

public class OKMStressGetContentTest {

    private static final int NR_OF_CLIENTS = 1000;
    private static final String DOC_ID = "<put in the docid of a document with a size of a least 50 or more megabytes>";

    private static final Logger LOG = LoggerFactory.getLogger(OKMStressGetContentTest.class);

    private static OKMWebservices okm;

    @BeforeClass
    public static void before() {
        okm = OKMWebservicesFactory.newInstance("https://demo.openkm.com/OpenKM", "user0", "pass0");
    }

    @Test
    public void testStress()
            throws Exception {

        stress(() -> {
            InputStream is;
            try {
                UUID uuid = UUID.randomUUID();
                LOG.info(uuid.toString() + " - start");
                is = okm.getContent(DOC_ID);
                LOG.info(uuid.toString() + " - end");
            } catch (RepositoryException | IOException | PathNotFoundException | AccessDeniedException | DatabaseException | UnknowException | WebserviceException e) {
                throw new RuntimeException(e);
            }
        }, NR_OF_CLIENTS);

    }

    private void stress(final Runnable action, int nrOfClients) throws InterruptedException {
        spawnThreads(action, nrOfClients).await();
    }

    private CountDownLatch spawnThreads(final Runnable action, int nrOfClients) {
        final CountDownLatch finished = new CountDownLatch(nrOfClients);
        ExecutorService executor = Executors.newFixedThreadPool(nrOfClients);

        for (int i = 0; i < nrOfClients; i++) {
            executor.execute(() -> {
                try {
                    action.run();
                } finally {
                    finished.countDown();
                }
            });
        }

        return finished;
    }

}
